/*---------------------------------------------------*
 |      FRegion.java                                 |
 |---------------------------------------------------|
 | Ce fichier dfinit les rgions de l'image qui     |
 | constitueront la fusion.                          |
 *---------------------------------------------------*/

// Dsignation du package d'appartenance
package photoshed.filtres.fusion;

// Importations des librairies JAVA ncssaires
import java.util.Vector;
import java.util.Enumeration;

// Dbut de la classe FRegion
public class FRegion
{
	// ----- Dclaration des attributs de FRegion
	public Integer label;
	public double moyenne;
	public int aire;
	public Vector composantes = new Vector(10, 5);
	public Vector voisins;

	// ----- Constructeurs
	public FRegion(Integer label)
	{
		this.label = label;
		this.moyenne = 0;
		this.aire = 0;
		this.voisins = new Vector(10, 5);
	}

	public FRegion(FRegion fr)
	{
		this.label = new Integer(fr.label.intValue());
		this.moyenne = fr.moyenne;
		this.aire = fr.aire;
		this.composantes = new Vector(10, 5);
		this.voisins = (Vector) fr.voisins.clone();
	}

	// ----- Mthodes d'instances

	// Procdure permettant d'ajouter un nouveau point  la FRegion
	public void ajouterPoint(int couleur)
	{
		this.moyenne += couleur;
		this.aire++;
	}

	// Procdure permettant d'ajouter un voisin
	public void ajouterVoisin(FRegion fr)
	{
		if (!(voisins.contains(fr)) && this != fr)
			voisins.add((Object) fr);
	}

	// Procdure permettant de finaliser notre FRegion aprs sa construction
	public void finaliserConstruction()
	{
		this.moyenne = this.moyenne / this.aire;
		trierVoisins();
	}

	// Procdure permettant d'ajouter une composante et de sauvegarder l'ensemble de ses donnes
	public void ajouterComposante(FRegion fr)
	{
		// Appropriation de l'ensemble des voisins viables de cette rgion
		Enumeration e;
		FRegion temp;
		for (e = fr.voisins.elements(); e.hasMoreElements();)
		{
			temp = (FRegion) e.nextElement();
			this.ajouterVoisin(temp);
			temp.remplacerVoisin(fr, this);
		}
		fr.voisins = null;
		trierVoisins();

		// Appropriation de la liste des rgions composantes
		for (e = fr.composantes.elements(); e.hasMoreElements();)
			composantes.add(e.nextElement());
		fr.composantes = null;

		// Appropriation des donnes de la rgion
		composantes.add((Object) fr.label);
		this.moyenne = (this.moyenne * this.aire) + (fr.moyenne * fr.aire);
		this.aire += fr.aire;
		this.moyenne = this.moyenne / this.aire;
	}

	// Procdure permettant de remplacer un voisin par un autre voisin
	public void remplacerVoisin(FRegion ancienne, FRegion nouvelle)
	{
		int index_ancienne = voisins.indexOf((Object) ancienne);
		if (index_ancienne >= 0)
			voisins.remove(index_ancienne);
		if (!(voisins.contains(nouvelle)) && this != nouvelle)
		{
			voisins.add((Object) nouvelle);
			trierVoisins();
		}
	}

	// Procdure permettant de trier les voisins selon les moyennes pas ordre croissant, et selon l'aire par ordre croissante
	public void trierVoisins()
	{
		// Tri de la structure
		FRegion max, current;
		int indice, fin = voisins.size() - 1, index;

		while (fin >= 1)
		{
			max = (FRegion) voisins.elementAt(0);
			index = 0;

			for (indice = 1; indice <= fin; indice++)
			{
				current = (FRegion) voisins.elementAt(indice);

				if (current.moyenne > max.moyenne || ((current.moyenne == max.moyenne) && (current.aire > max.aire)))
				{
					max = current;
					index = indice;
				}
			}
			current = (FRegion) voisins.remove(fin);
			voisins.insertElementAt(max, fin);
			voisins.set(index, current);
			fin--;
		}
	}
}